/*(C) July, 2005 Muthiah Annamalai, G2# bindings. 
 * MONO bindings to G2 library.
 * This program is free software.
 * Code may be used or distributed under GPL.
 * (C) July 2005 Muthiah Annamalai.
*/

using System;
using System.Diagnostics;
using System.Runtime.InteropServices;

namespace Mono.G2
{
public class G2
{
private int device_id;
private DeviceType type_id;
private string filename;
private bool open=false;

public enum DeviceType{ FIG, GD, VD, PS, EPSF, X11, WIN} ;
public int x;
public static string [] DeviceMap =  {"FIG","GD","VD","PS","EPS","X11","WIN"};
public enum GDType { jpeg =  0 ,  png =  1};
public enum QPShape{ QPrect =  0 , QPcirc =  1 };

public enum PSPaper {A0, A1 =  1, A2 =  2 , A3 =  3 , A4 =  4 , A5 =  5 ,
	     A6 =  6 , A7 =  7 , A8 =  8 , A9 =  9 , B0 =  10 ,B1 =  11 ,
	     B2 =  12 ,B3 =  13 , B4 =  14 ,  B5 =  15 , B6 =  16 ,
	     B7 =  17 ,B8 =  18 , B9 =  19 , B10 =  20 , 
	     Comm_10_Envelope =  21 , C5_Envelope =  22 , 
	     DL_Envelope =  23 ,  Folio =  24 ,
	     Executive =  25 ,Letter =  26 , Legal =  27 , Ledger =  28 ,
	     Tabloid =  29 } ;

public enum PSOrientation { Landscape =  0 , Portrait =  1 } ;
  
public enum PSFormat{ PostScript =  0 , EPSF =  1 , 
		      EPSFCLIP =  2 } ;

public int DeviceID
  {
    get
      {
	return this.device_id;
      }
  }

public G2(int id,DeviceType enumval, bool open,string filen)
  {
    filename =filen;
    device_id = id;
    type_id = enumval;
    open=open;
  }

public G2(int id,DeviceType enumval,bool openf)
  {
    filename =null;
    device_id = id;
    type_id = enumval;
    open=openf;
  }

  /* All 3 share same constructor */
  [DllImport("libg2.so.0.0.70")]
  static extern int  g2_open_FIG ( string file_name ) ;

  [DllImport("libg2.so.0.0.70")]
  static extern int  g2_open_EPSF (  string file_name ) ;

  [DllImport("libg2.so.0.0.70")]
  static extern int  g2_open_vd ( ) ;

public G2(DeviceType enumval,string filename)
  {
    int id=-1;
    this.filename = filename;
    this.type_id = enumval;
    if ( filename == null && enumval !=DeviceType.VD )
      {
	throw new Exception("Cannot Open Device " +
			    G2.DeviceMap[(int)enumval] +
			    " without a filename");
      }

    switch(enumval)
      {
      case DeviceType.FIG:
	id=g2_open_FIG(filename);
	break;
      case DeviceType.EPSF:
	id=g2_open_EPSF(filename);
	break;
      case DeviceType.VD:
	id=g2_open_vd();
	break;
      default:
	throw new Exception("Cannot Open Device  Unknown device");
      };
    this.device_id=id;
    this.open=true;
  }


  [DllImport("libg2.so.0.0.70")]
  static extern int  g2_open_gd (  string filename, int width,
				   int height, int gd_type ) ;

public G2(GDType type,string filename,
	  int width,int height)
  {
    int id=-1;
    Debug.Assert(filename != null);
    id = g2_open_gd(filename,width,height,(int)type);
    this.filename =filename;
    this.device_id = id;
    this.type_id = DeviceType.GD;
    this.open=true;
  }

  [DllImport("libg2.so.0.0.70")]
  static extern int  g2_open_PS (  string file_name,
				   int paper,
				   int orientation ) ;

public G2(PSPaper psize, PSOrientation psorient,
	  string filen)
  {
    int id=-1;
    Debug.Assert(filename != null);
    id = g2_open_PS(filename,psize,psorient);
    filename =filen;
    device_id = id;
    type_id = DeviceType.PS;
    open=true;
  }


  /* these 2 can share constructor */
  [DllImport("libg2.so.0.0.70")]
  static extern int  g2_open_EPSF_CLIP (string file_name, long width, long height ) ;

  [DllImport("libg2.so.0.0.70")]
  static extern int  g2_open_X11 (  int width, int height ) ;

public G2(DeviceType enumval,int width,int height,string filen)
  {

    int id=-1;
    filename=filen;
    if (filename == null && enumval != DeviceType.X11)
      {
	throw new Exception("Cannot Open Device " +
			    G2.DeviceMap[(int)enumval] +" without a filename");
      }
    
    switch(enumval)
      {
      case DeviceType.EPSF:
	id=g2_open_EPSF_CLIP(filename,width,height);
	break;
      case DeviceType.X11:
	id=g2_open_X11(width,height);
	break;
      default:
	throw new Exception("Unknown Enumerator Type: No such device");
      }
    filename=filen;
    device_id=id;    
    type_id = enumval;
    open=true;
  }

public G2(DeviceType enumval,int width,int height)
  {
    int id=-1;
    Debug.Assert(enumval==DeviceType.X11);
    id = g2_open_X11(width,height);    
    filename =null;
    device_id = id;
    type_id = DeviceType.PS;
    open=true;
  }


  [DllImport("libg2.so.0.0.70")]
  static extern  void g2_close (  int dev ) ;
public void Close()
  {
    g2_close(this.DeviceID);
    this.device_id=-1;
    this.open=false;
    return;
  }
  
  [DllImport("libg2.so.0.0.70")]
  static extern  void g2_set_auto_flush (  int dev, bool on_off ) ;
public  bool  SetAutoFlush
  {
    set
      {
	g2_set_auto_flush(this.DeviceID,value);
      }
  }

  [DllImport("libg2.so.0.0.70")]
  static extern  void g2_flush (  int dev ) ;
public void Flush()
  {
    g2_flush(this.DeviceID);
    return;
  }
  
  [DllImport("libg2.so.0.0.70")]
  static extern  void g2_save (  int dev ) ;
public void Save()
  {
    g2_save(this.DeviceID);
    return;
  }

  [DllImport("libg2.so.0.0.70")]
  static extern  void g2_set_coordinate_system (  int dev, double x_origin, double y_origin, double x_mul, double y_mul ) ;

public void SetCoordinateSystem(double x_origin, double y_origin,
				double x_mul,double y_mul)
  {
    g2_set_coordinate_system(this.DeviceID,x_origin,y_origin,x_mul,y_mul);
    return;
  }
  


  [DllImport("libg2.so.0.0.70")]
  static extern  int g2_ld (    ) ;
  [DllImport("libg2.so.0.0.70")]
  static extern  void g2_set_ld (  int dev ) ;

public int Ld
  {
    get
      {
	return g2_ld();
      }
    set
      {
	g2_set_ld(value);
      }
  }


  
  [DllImport("libg2.so.0.0.70")]
  static extern int g2_ink (int pd_dev, double red, double green,double blue );

  public int Ink( double red, double green, double blue )
  {
    int rtype=g2_ink(this.DeviceID, red, green, blue);
    return rtype;
  }
  
  [DllImport("libg2.so.0.0.70")]
  static extern  void g2_pen (  int dev, int color ) ;
public void Pen( int color ) 
  {
    g2_pen(this.DeviceID,color);
    return;
  }

  [DllImport("libg2.so.0.0.70")]
  static extern  void g2_set_dash (  int dev, int N, double [] dashes ) ;
public double [] Dash
  {
    set
      {
	g2_set_dash(this.DeviceID,value.Length,value);
      }
  }

  [DllImport("libg2.so.0.0.70")]
  static extern  void g2_set_font_size (  int dev, double size ) ;
public double FontSize
  {
    set
      {
	g2_set_font_size(this.DeviceID,value);
      }
  }

  [DllImport("libg2.so.0.0.70")]
  static extern  void g2_set_line_width (  int dev, double w ) ;
public double LineWidth
  {
    set
      {
	g2_set_line_width(this.DeviceID,value);
      }
  }

  [DllImport("libg2.so.0.0.70")]
  static extern  void g2_clear_palette (  int dev ) ;
  public void ClearPalette()
  {
    g2_clear_palette(this.DeviceID);
  }
  
  [DllImport("libg2.so.0.0.70")]
  static extern  void g2_reset_palette (  int dev ) ;
public void ResetPalette()
  {
    g2_reset_palette(this.DeviceID);
  }

  [DllImport("libg2.so.0.0.70")]
  static extern  void g2_allocate_basic_colors (  int dev ) ;
public void AllocateBasicColors()
  {
    g2_allocate_basic_colors (this.DeviceID);
  }

  [DllImport("libg2.so.0.0.70")]
  static extern  void g2_clear (  int dev ) ;
  public void Clear()
  {
    g2_clear (this.DeviceID);
  }
  
  [DllImport("libg2.so.0.0.70")]
  static extern  void g2_set_background (  int dev, int color ) ;
public int Background
  {
    set
      {
        g2_set_background (this.DeviceID, value) ;
      }
  }

  [DllImport("libg2.so.0.0.70")]
  static extern  void g2_move (  int dev, double x, double y ) ;
public void Move( double x, double y ) 
  {
    g2_move (this.DeviceID,x,y);
  }

  [DllImport("libg2.so.0.0.70")]
  static extern  void g2_move_r (  int dev, double dx, double dy ) ;
public void MoveR( double dx, double dy ) 
  {
    g2_move_r (this.DeviceID,dx,dy);
  }

  [DllImport("libg2.so.0.0.70")]
  static extern  void g2_plot (  int dev, double x, double y ) ;
public void Plot( double dx, double dy ) 
  {
    g2_plot (this.DeviceID,dx,dy);
  }

  [DllImport("libg2.so.0.0.70")]
  static extern  void g2_plot_r (  int dev, double dx, double dy ) ;
public void PlotR( double dx, double dy ) 
  {
    g2_plot_r (this.DeviceID,dx,dy);
  }


  [DllImport("libg2.so.0.0.70")]
  static extern  void g2_line (  int dev, double x1, double y1,
				 double x2, double y2 ) ;
  
public void Line( double x1, double y1,
		  double x2, double y2 ) 
  {
    g2_line (this.DeviceID,x1,y1,x2,y2);
  }

  [DllImport("libg2.so.0.0.70")]
  static extern  void g2_line_r (  int dev, double dx, double dy ) ;
public void LineR( double x1, double y1)
  {
    g2_line_r (this.DeviceID,x1,y1);
  }
  
  [DllImport("libg2.so.0.0.70")]
  static extern  void g2_line_to (  int dev, double x, double y ) ;
public void LineTo( double x1, double y1)
  {
    g2_line_to (this.DeviceID,x1,y1);
  }


  [DllImport("libg2.so.0.0.70")]
  static extern  void g2_poly_line (  int dev, int N_pt, double [] points ) ;
public void PolyLine( double [] points)
  {
    g2_poly_line (this.DeviceID,points.Length,points);
  }

  [DllImport("libg2.so.0.0.70")]
  static extern  void g2_triangle (  int dev, double x1, 
				     double y1, double x2,
				     double y2, double x3, double y3 ) ;
public void Triangle( double x1, double y1,
		      double x2, double y2 ,
		      double x3, double y3 ) 
  {
    g2_triangle (this.DeviceID,x1,y1,x2,y2,x3,y3);
  }

  [DllImport("libg2.so.0.0.70")]
  static extern  void g2_filled_triangle (  int dev, double x1, 
					    double y1, double x2,
					    double y2, double x3, double y3 );
public void FilledTriangle( double x1, double y1,
			    double x2, double y2 ,
			    double x3, double y3 ) 
  {
    g2_filled_triangle (this.DeviceID,x1,y1,x2,y2,x3,y3);
  }

  [DllImport("libg2.so.0.0.70")]
  static extern  void g2_rectangle (  int dev, double x1,
				      double y1, double x2, double y2 ) ;
public void Rectangle( double x1, double y1,
		       double x2, double y2 ) 
  {
    g2_rectangle (this.DeviceID,x1,y1,x2,y2);
  }
  
  [DllImport("libg2.so.0.0.70")]
  static extern  void g2_filled_rectangle (  int dev, double x1,
					     double y1, double x2,
					     double y2 ) ;
public void FilledRectangle( double x1, double y1,
			     double x2, double y2 ) 
  {
    g2_filled_rectangle (this.DeviceID,x1,y1,x2,y2);
  }

  [DllImport("libg2.so.0.0.70")]
  static extern  void g2_polygon (  int dev, int N_pt, double [] points ) ;
public void Polygon( double [] points)
  {
    g2_polygon (this.DeviceID,points.Length,points);
  }

  [DllImport("libg2.so.0.0.70")]
  static extern  void g2_filled_polygon (  int dev, int N_pt,
					   double [] points ) ;
public void FilledPolygon( double [] points)
  {
    g2_filled_polygon (this.DeviceID,points.Length,points);
  }


  [DllImport("libg2.so.0.0.70")]
  static extern  void g2_circle (  int dev, double x, double y, double r ) ; 
public void Circle( double x, double y, double r ) 
  {
    g2_circle (this.DeviceID,x,  y,  r ) ;
  }
 

  [DllImport("libg2.so.0.0.70")]
  static extern  void g2_filled_circle (  int dev, double x,
					  double y, double r ) ;
public void FilledCircle( double x, double y, double r ) 
  {
    g2_filled_circle (this.DeviceID,x,  y,  r ) ;
  }

  [DllImport("libg2.so.0.0.70")]
  static extern  void g2_ellipse (  int dev, double x, double y, double r1, double r2 ) ;
public void Ellipse( double x, double y, double r1,double r2 ) 
  {
    g2_ellipse (this.DeviceID,x,  y,  r1,r2 ) ;
  }

  [DllImport("libg2.so.0.0.70")]
  static extern  void g2_filled_ellipse (  int dev, double x, double y, double r1, double r2 ) ;
public void FilledsEllipse( double x, double y, double r1,double r2 ) 
  {
    g2_filled_ellipse (this.DeviceID,x,  y,  r1,r2 ) ;
  }

  [DllImport("libg2.so.0.0.70")]
  static extern  void g2_arc (  int dev, double x, double y, double r1, double r2, double a1, double a2 ) ;
public void Arc( double x, double y, double r1, double r2, double a1, double a2 )
  {
    g2_arc (this.DeviceID,  x,  y,  r1,  r2,  a1,  a2 );
    return;
  }
  
  [DllImport("libg2.so.0.0.70")]
  static extern  void g2_filled_arc (  int dev, double x, double y, double r1, double r2, double a1, double a2 ) ;
public void FilledArc( double x, double y, double r1, double r2, double a1, double a2 )
  {
    g2_filled_arc (this.DeviceID,  x,  y,  r1,  r2,  a1,  a2 );
    return;
  }
  
  [DllImport("libg2.so.0.0.70")]
  static extern  void g2_string (  int dev, double x, double y, string text ) ;
public void String(double x, double y, string text )
  {
    g2_string(this.DeviceID,x,y,text);
    return;
  }

  [DllImport("libg2.so.0.0.70")]  
  static extern  void g2_image (  int dev, double x, double y,
				  int x_size, int y_size, int [] pens ) ;
public void Image(double x, double y, int x_size, int y_size, int [] pens )
  {
    g2_image(this.DeviceID,x,y,x_size,y_size,pens);
    return;
  }

  [DllImport("libg2.so.0.0.70")]
  static extern  void g2_set_QP (  int dev, double d, QPShape shape ) ;
public  void SetQP (  int dev, double d, QPShape shape ) 
  {
    g2_set_QP (  dev, d, shape ) ;
    return;
  }
  [DllImport("libg2.so.0.0.70")]
  static extern  void g2_plot_QP (  int dev, double x, double y ) ;
public  void PlotQP (  int dev, double x, double y ) 
  {
    g2_plot_QP (  dev, x, y ) ;
    return;
  }
  [DllImport("libg2.so.0.0.70")]
  static extern  void g2_query_pointer (  int dev, double [] x, double [] y, IntPtr button ) ;
public  int QueryPointer (  int dev, double [] x, double [] y) 
  {
    int button=0;
    g2_query_pointer (  dev, x, y,(IntPtr) button ) ;
    return button;
  }
  [DllImport("libg2.so.0.0.70")]
  static extern  void g2_spline (  int id, int n, double [] points, int o ) ;
public  void Spline ( double [] points, int o ) 
  {
    g2_spline (this.DeviceID, points.Length, points, o ) ;
    return;
  }
  [DllImport("libg2.so.0.0.70")]
  static extern  void g2_b_spline ( int  id, int n, double [] points, int o ) ;
public  void BSpline ( double [] points, int o ) 
  {
    g2_b_spline (this.DeviceID,points.Length, points, o ) ;
    return;
  }
  [DllImport("libg2.so.0.0.70")]
  static extern  void g2_raspln (  int id, int n, double [] points, double tn ) ;
public  void Raspln ( double [] points, double tn ) 
  {
    g2_raspln (this.DeviceID,points.Length, points, tn ) ;
    return;
  }
  [DllImport("libg2.so.0.0.70")]
  static extern  void g2_para_3 (  int id, int n, double [] points ) ;
public  void Para3 ( double [] points ) 
  {
    g2_para_3 (this.DeviceID,points.Length, points ) ;
    return;
  }
  [DllImport("libg2.so.0.0.70")]
  static extern  void g2_para_5 (  int id, int n, double [] points ) ;
public  void Para5 (double [] points ) 
  {
    g2_para_5 ( this.DeviceID, points.Length, points ) ;
    return;
  }
  [DllImport("libg2.so.0.0.70")]
  static extern  void g2_filled_spline (  int id, int n, double [] points, int o ) ;
public  void FilledSpline ( double [] points, int o ) 
  {
    g2_filled_spline ( this.DeviceID, points.Length, points, o ) ;
    return;
  }
  [DllImport("libg2.so.0.0.70")]
  static extern  void g2_filled_b_spline (  int id, int n, double [] points, int o ) ;
public  void FilledBSpline (double [] points, int o ) 
  {
    g2_filled_b_spline ( this.DeviceID, points.Length, points, o ) ;
    return;
  }
  [DllImport("libg2.so.0.0.70")]
  static extern  void g2_filled_raspln (  int id, int n, double [] points, double tn ) ;
public  void FilledRaspln (  double [] points, double tn ) 
  {
    g2_filled_raspln (  this.DeviceID, points.Length, points, tn ) ;
    return;
  }
  [DllImport("libg2.so.0.0.70")]
  static extern  void g2_filled_para_3 (  int id, int n, double [] points ) ;
public  void FilledPara3 ( double [] points ) 
  {
    g2_filled_para_3 (  this.DeviceID,points.Length, points ) ;
    return;
  }
  [DllImport("libg2.so.0.0.70")]
  static extern  void g2_filled_para_5 (  int id, int n, double [] points ) ;
public  void FilledPara5 (  double [] points ) 
  {
    g2_filled_para_5 (this.DeviceID,points.Length, points ) ;
    return;
  }
  [DllImport("libg2.so.0.0.70")]
  static extern  int g2_device_exist (  int dix ) ;
public static int DeviceExist (G2 obj ) 
  {
    int rtype;
    rtype = 
      g2_device_exist ( obj.DeviceID ) ;
    return rtype;
  }

  /* //LEAVING HEADACHE for me
  [DllImport("libg2.so.0.0.70")]
  static extern  void g2_get_pd_handles (  int pd, IntPtr [] handles);
public  int [] GetPdHandles
  {
    get
      {
	int [] handles=new int [32];
	for (int i=0;i<32;i++)
	  handles[i]=0;

	g2_get_pd_handles (this.DeviceID,ref handles) ;
	return handles;
      }
  }
  */

  [DllImport("libg2.so.0.0.70")]
  static extern  int g2_open_PS (  string file_name, PSPaper paper,
				   PSOrientation orientation ) ;
public static G2 OpenPS (  string file_name, PSPaper paper,
		     PSOrientation orientation ) 
  {
    int rtype;
    rtype = 
      g2_open_PS (  file_name, paper, orientation ) ;
 
    return new G2(rtype,DeviceType.PS,true);
  }

public static G2 OpenGd (  string filename, int width,
			   int height, GDType type)
  {
    return new  G2(type,filename,width, height);
  }

  /*
  [DllImport("libg2.so.0.0.70")]
  static extern  int g2_open_EPSF (  string file_name ) ;
  */
public  static G2 OpenEPSF (  string file_name ) 
  {
    int rtype;
    rtype =  g2_open_EPSF (file_name) ;
    return new G2(rtype,DeviceType.EPSF,true,file_name);
  }

  /*
  [DllImport("libg2.so.0.0.70")]
  static extern  int g2_open_EPSF_CLIP (  string file_name,
					  long width, long height ) ;
  */
  public  static G2 OpenEPSFCLIP (  string file_name, 
				    long width, long height ) 
  {
    int rtype;
    rtype = g2_open_EPSF_CLIP (  file_name, width, height ) ;
    return new G2(rtype,DeviceType.EPSF,true,file_name);
  }

  /*
  [DllImport("libg2.so.0.0.70")]
  static extern  int g2_open_X11 (  int width, int height ) ;
  */
  public  static G2 OpenX11 (  int width, int height ) 
  {
    int rtype;
    rtype = g2_open_X11 (  width, height ) ;
    return new G2(rtype,DeviceType.X11,true);
  }

  [DllImport("libg2.so.0.0.70")]
  static extern  int g2_open_X11X (  int width, int height,
				     int x, int y, string window_name,
				     string icon_name, string icon_data, 
				     int icon_width, int icon_height ) ;
    public static G2 OpenX11X (  int width, int height,
			   int x, int y, string window_name, 
			   string icon_name, string icon_data,
			   int icon_width, int icon_height) 
  {
    int rtype;
    rtype =  g2_open_X11X (  width, height, x, y, window_name, 
			     icon_name, icon_data, icon_width, icon_height ) ;
    return new  G2(rtype,DeviceType.X11,true);
  }

public static G2 OpenX11X (  int width, int height,
			     int x, int y, string window_name)
  {
    return OpenX11X(width,height,x,y,window_name,null,null,0,0);
  }

  public static G2 OpenVd ()
  {
    int rtype;
    rtype =  g2_open_vd();
    return new  G2(rtype,DeviceType.VD,true);
  }

  public static G2 OpenFIG (string filename)
  {
    int rtype;
    rtype =  g2_open_FIG(filename);	
    return new  G2(rtype,DeviceType.VD,true,filename);
  }


  [DllImport("libg2.so.0.0.70")]
  static extern void g2_detach (int vd_dev,int dev );
  public void Detach(G2 obj)
  {
    g2_detach(this.DeviceID,obj.DeviceID);
    return;
  }

  [DllImport("libg2.so.0.0.70")]
  static extern void  g2_attach (  int vd_dev, int dev ) ;
  public void Attach(G2 obj)
  {
    g2_attach(this.DeviceID,obj.DeviceID);
    return;
  }

}
}

